@@ -75,7 +75,7 @@ pub struct Cli {
     #[arg(short, long, group = "operations", value_name = "FILE")]
     pub render: Option<PathBuf>,
 
-    /// Clean the cache.
+    /// Interactively delete contents of the cache directory.
     #[arg(long, group = "operations")]
     pub clean_cache: bool,
 
@@ -127,6 +127,10 @@ pub struct Cli {
     #[arg(short, long)]
     pub quiet: bool,
 
+    /// Be more verbose (can be specified twice).
+    #[arg(long, action = ArgAction::Count)]
+    pub verbose: u8,
+
     /// Specify when to enable color.
     #[arg(long, value_name = "WHEN", default_value_t = ColorChoice::default())]
     pub color: ColorChoice,
