@@ -1,10 +1,12 @@
+use std::cmp;
 use std::collections::{BTreeMap, HashMap};
 use std::ffi::OsString;
 use std::fs::{self, File};
-use std::io::{self, BufWriter, Cursor, Write};
+use std::io::{self, BufRead, BufWriter, Cursor, Write};
 use std::path::{Path, PathBuf};
 use std::time::Duration;
 
+use log::{debug, info, warn};
 use once_cell::unsync::OnceCell;
 use ureq::tls::{RootCerts, TlsConfig};
 use yansi::Paint;
@@ -12,10 +14,12 @@ use zip::ZipArchive;
 
 use crate::config::Config;
 use crate::error::{Error, Result};
-use crate::util::{self, info_end, info_start, infoln, warnln, Dedup};
+use crate::util::{self, info_end, info_start, Dedup};
 
 pub const ENGLISH_DIR: &str = "pages.en";
+const CHECKSUM_FILE: &str = "tldr.sha256sums";
 const USER_AGENT: &str = concat!(env!("CARGO_PKG_NAME"), '/', env!("CARGO_PKG_VERSION"));
+const HTTP_TIMEOUT: Option<Duration> = Some(Duration::from_secs(10));
 
 type PagesArchive = ZipArchive<Cursor<Vec<u8>>>;
 
@@ -83,7 +87,10 @@ impl<'a> Cache<'a> {
     ) -> Result<BTreeMap<String, PagesArchive>> {
         let agent = ureq::Agent::config_builder()
             .user_agent(USER_AGENT)
-            .timeout_global(Some(Duration::from_secs(30)))
+            // The global timeout isn't set, because it prevents some people from downloading
+            // page archives. See https://github.com/tldr-pages/tlrc/issues/131.
+            .timeout_resolve(HTTP_TIMEOUT)
+            .timeout_connect(HTTP_TIMEOUT)
             .tls_config(
                 TlsConfig::builder()
                     .root_certs(RootCerts::PlatformVerifier)
@@ -92,11 +99,12 @@ impl<'a> Cache<'a> {
             .build()
             .into();
 
-        let sums = Self::get_asset(&agent, &format!("{mirror}/tldr.sha256sums"))?;
+        let sums = Self::get_asset(&agent, &format!("{mirror}/{CHECKSUM_FILE}"))?;
         let sums_str = String::from_utf8_lossy(&sums);
         let sum_map = Self::parse_sumfile(&sums_str)?;
+        debug!("sum file parsed, available languages: {:?}", sum_map.keys());
 
-        let old_sumfile_path = self.dir.join("tldr.sha256sums");
+        let old_sumfile_path = self.dir.join(CHECKSUM_FILE);
         let old_sums = fs::read_to_string(&old_sumfile_path).unwrap_or_default();
         let old_sum_map = Self::parse_sumfile(&old_sums).unwrap_or_default();
 
@@ -105,13 +113,13 @@ impl<'a> Cache<'a> {
         for lang in languages {
             let lang = &**lang;
             let Some(sum) = sum_map.get(lang) else {
-                // Skip nonexistent languages.
+                debug!("'{lang}': language not available, skipping it");
                 continue;
             };
 
             let lang_dir = format!("pages.{lang}");
             if Some(sum) == old_sum_map.get(lang) && self.subdir_exists(&lang_dir) {
-                infoln!("'pages.{lang}' is up to date");
+                info!("'{lang_dir}' is up to date");
                 continue;
             }
 
@@ -128,7 +136,7 @@ impl<'a> Cache<'a> {
                 )));
             }
 
-            info_end!(" {}", "OK".green().bold());
+            info_end!("{}", "OK".green().bold());
 
             langdir_archive_map.insert(lang_dir, ZipArchive::new(Cursor::new(archive))?);
         }
@@ -187,7 +195,7 @@ impl<'a> Cache<'a> {
         for i in 0..archive.len() {
             let mut zipfile = archive.by_index(i)?;
             let Some(fname) = zipfile.enclosed_name() else {
-                warnln!(
+                debug!(
                     "found an unsafe path in the zip archive: '{}', ignoring it",
                     zipfile.name()
                 );
@@ -235,7 +243,7 @@ impl<'a> Cache<'a> {
         let archives = self.download_and_verify(mirror, languages)?;
 
         if archives.is_empty() {
-            infoln!(
+            info!(
                 "there is nothing to do. Run 'tldr --clean-cache' if you want to force an update."
             );
             return Ok(());
@@ -266,7 +274,7 @@ impl<'a> Cache<'a> {
             }
         }
 
-        infoln!(
+        info!(
             "cache update successful (total: {} pages, {} new).",
             all_downloaded.green().bold(),
             all_new.green().bold(),
@@ -275,17 +283,38 @@ impl<'a> Cache<'a> {
         Ok(())
     }
 
-    /// Delete the cache directory.
+    /// Interactively delete contents of the cache directory.
     pub fn clean(&self) -> Result<()> {
-        if !self.dir.is_dir() {
-            infoln!("cache does not exist, not cleaning.");
+        if !self.dir.is_dir() || fs::read_dir(self.dir).map(|mut rd| rd.next().is_none())? {
+            info!("cache does not exist, not cleaning.");
             fs::create_dir_all(self.dir)?;
             return Ok(());
         }
 
-        infoln!("cleaning the cache directory...");
-        fs::remove_dir_all(self.dir)?;
-        fs::create_dir_all(self.dir)?;
+        let sumfile = self.dir.join(CHECKSUM_FILE);
+        if sumfile.is_file() {
+            info!("removing '{}'...", sumfile.display().red());
+            fs::remove_file(sumfile)?;
+        }
+
+        let mut stdout = io::stdout().lock();
+        let mut stdin = io::stdin().lock();
+        let mut resp = String::new();
+
+        for dir in fs::read_dir(self.dir)? {
+            let dir = dir?.path();
+
+            write!(stdout, "Remove '{}'? [y/N] ", dir.display().red())?;
+            stdout.flush()?;
+            stdin.read_line(&mut resp)?;
+
+            if resp.starts_with(['y', 'Y']) {
+                info!("removing...");
+                fs::remove_dir_all(dir)?;
+            }
+
+            String::clear(&mut resp);
+        }
 
         Ok(())
     }
@@ -312,6 +341,7 @@ impl<'a> Cache<'a> {
                     // read_dir() order can differ across runs, so it's
                     // better to sort the Vec for consistency.
                     result.sort_unstable();
+                    debug!("found platforms: {result:?}");
                     Ok(result)
                 }
             })
@@ -341,7 +371,9 @@ impl<'a> Cache<'a> {
         for lang_dir in lang_dirs {
             let path = self.dir.join(lang_dir).join(&platform).join(fname);
 
+            debug!("trying path: {path:?}");
             if path.is_file() {
+                debug!("page found");
                 return Some(path);
             }
         }
@@ -355,6 +387,7 @@ impl<'a> Cache<'a> {
 
         let platforms = self.get_platforms_and_check(platform)?;
         let file = format!("{name}.md");
+        debug!("searching for page: '{file}'");
 
         let mut result = vec![];
         let mut lang_dirs: Vec<String> = languages.iter().map(|x| format!("pages.{x}")).collect();
@@ -386,12 +419,12 @@ impl<'a> Cache<'a> {
                     let alt_platform = alt_platform.to_string_lossy();
 
                     if platform == "common" {
-                        warnln!(
+                        warn!(
                             "showing page from platform '{alt_platform}', \
                             because '{name}' does not exist in 'common'"
                         );
                     } else {
-                        warnln!(
+                        warn!(
                             "showing page from platform '{alt_platform}', \
                             because '{name}' does not exist in '{platform}' and 'common'"
                         );
@@ -402,6 +435,7 @@ impl<'a> Cache<'a> {
             }
         }
 
+        debug!("found {} page(s)", result.len());
         Ok(result)
     }
 
@@ -554,17 +588,19 @@ impl<'a> Cache<'a> {
         }
 
         writeln!(stdout, "Installed languages:")?;
+        let width = cmp::max(n_map.keys().map(String::len).max().unwrap(), 5);
+        //                                  "total" is 5 characters long. ^^
 
         for (lang, n) in n_map {
-            writeln!(
-                stdout,
-                // Language codes are at most 5 characters (ll_CC).
-                "{lang:5} : {}",
-                n.green().bold(),
-            )?;
+            writeln!(stdout, "{lang:width$} : {}", n.green().bold())?;
         }
 
-        writeln!(stdout, "total : {} pages", n_total.green().bold())?;
+        writeln!(
+            stdout,
+            "{:width$} : {} pages",
+            "total",
+            n_total.green().bold()
+        )?;
 
         Ok(())
     }
@@ -573,7 +609,7 @@ impl<'a> Cache<'a> {
     pub fn age(&self) -> Result<Duration> {
         self.age
             .get_or_try_init(|| {
-                let sumfile = self.dir.join("tldr.sha256sums");
+                let sumfile = self.dir.join(CHECKSUM_FILE);
                 let metadata = if sumfile.is_file() {
                     fs::metadata(&sumfile)
                 } else {
@@ -592,3 +628,137 @@ impl<'a> Cache<'a> {
             .copied()
     }
 }
+
+#[cfg(test)]
+mod tests {
+    use super::*;
+    use tempfile::{tempdir, TempDir};
+
+    /// Create a temporary cache dir for tests with the specified pages.
+    fn prepare(pages: &[&str]) -> TempDir {
+        let cachedir = tempdir().unwrap();
+        let d = cachedir.path();
+
+        for p in pages {
+            let mut page_dir = Path::new(p).components();
+            page_dir.next_back();
+            fs::create_dir_all(d.join(page_dir)).unwrap();
+            File::create(d.join(p)).unwrap();
+        }
+
+        cachedir
+    }
+
+    #[test]
+    fn not_found() {
+        let tmpdir = prepare(&["pages.en/common/b.md", "pages.en/linux/b.md"]);
+        let c = Cache::new(tmpdir.path());
+        let pages = c.find("a", &["en".to_string()], "common").unwrap();
+        assert!(pages.is_empty());
+    }
+
+    #[test]
+    #[should_panic = "platform 'some_platform' does not exist"]
+    fn platform_does_not_exist() {
+        let tmpdir = prepare(&["pages.en/common/b.md", "pages.en/linux/b.md"]);
+        let c = Cache::new(tmpdir.path());
+        c.find("a", &["en".to_string()], "some_platform").unwrap();
+    }
+
+    #[test]
+    fn platform_priority() {
+        let tmpdir = prepare(&[
+            "pages.en/common/a.md",
+            "pages.en/linux/a.md",
+            "pages.en/osx/b.md",
+        ]);
+        let c = Cache::new(tmpdir.path());
+
+        let pages_common = c.find("a", &["en".to_string()], "common").unwrap();
+        let pages_linux = c.find("a", &["en".to_string()], "linux").unwrap();
+        let pages_osx = c.find("a", &["en".to_string()], "osx").unwrap();
+
+        assert_eq!(pages_common, pages_osx);
+        assert_eq!(pages_common.len(), 2);
+        assert!(pages_common[0].ends_with("pages.en/common/a.md"));
+        assert!(pages_common[1].ends_with("pages.en/linux/a.md"));
+
+        assert_eq!(pages_linux.len(), 2);
+        assert!(pages_linux[0].ends_with("pages.en/linux/a.md"));
+        assert!(pages_linux[1].ends_with("pages.en/common/a.md"));
+    }
+
+    #[test]
+    #[allow(clippy::similar_names)]
+    fn lang_priority() {
+        let tmpdir = prepare(&[
+            "pages.en/common/a.md",
+            "pages.xy/common/a.md",
+            "pages.en/common/b.md",
+            "pages.en/linux/c.md",
+        ]);
+        let c = Cache::new(tmpdir.path());
+
+        let pages_a_en = c
+            .find("a", &["en".to_string(), "xy".to_string()], "linux")
+            .unwrap();
+        let pages_a_xy = c
+            .find("a", &["xy".to_string(), "en".to_string()], "common")
+            .unwrap();
+
+        assert_eq!(pages_a_en.len(), 1);
+        assert_eq!(pages_a_xy.len(), 1);
+
+        assert!(pages_a_en[0].ends_with("pages.en/common/a.md"));
+        assert!(pages_a_xy[0].ends_with("pages.xy/common/a.md"));
+
+        let pages_b_xy = c
+            .find("b", &["xy".to_string(), "en".to_string()], "common")
+            .unwrap();
+
+        assert_eq!(pages_b_xy.len(), 1);
+        assert!(pages_b_xy[0].ends_with("pages.en/common/b.md"));
+    }
+
+    #[test]
+    fn list_pages() {
+        let tmpdir = prepare(&[
+            "pages.en/common/a.md",
+            "pages.en/common/b.md",
+            "pages.en/linux/c.md",
+            "pages.en/osx/d.md",
+            "pages.xy/linux/e.md",
+        ]);
+        let c = Cache::new(tmpdir.path());
+
+        let mut list = c.list_dir("common", "pages.en").unwrap();
+        list.sort_unstable();
+        assert_eq!(list, vec!["a.md", "b.md"]);
+
+        let mut list = c.list_dir("linux", "pages.en").unwrap();
+        list.sort_unstable();
+        assert_eq!(list, vec!["c.md"]);
+
+        let mut list = c.list_all_vec("pages.en").unwrap();
+        list.sort_unstable();
+        assert_eq!(list, vec!["a.md", "b.md", "c.md", "d.md"]);
+    }
+
+    #[test]
+    fn list_platforms() {
+        let tmpdir = prepare(&[
+            "pages.en/common/a.md",
+            "pages.en/linux/a.md",
+            "pages.en/osx/a.md",
+        ]);
+        let c = Cache::new(tmpdir.path());
+        assert_eq!(c.get_platforms().unwrap(), &["common", "linux", "osx"]);
+    }
+
+    #[test]
+    fn parse_sumfile() {
+        let s = "xyz    pages.en.zip\nzyx   pages.xy.zip\nabc   someotherfile\ncba  index.json";
+        let map = HashMap::from([("en", "xyz"), ("xy", "zyx")]);
+        assert_eq!(Cache::parse_sumfile(s).unwrap(), map);
+    }
+}
