@@ -1,11 +1,13 @@
-use std::fmt::{self, Display};
-use std::io::{self, Write};
+use std::fmt::Display;
+use std::io;
 use std::path::Path;
 use std::process::ExitCode;
 use std::result::Result as StdResult;
 
+use log::error;
 use yansi::Paint;
 
+#[derive(Debug)]
 pub enum ErrorKind {
     ParseToml,
     ParsePage,
@@ -14,6 +16,7 @@ pub enum ErrorKind {
     Other,
 }
 
+#[derive(Debug)]
 pub struct Error {
     pub kind: ErrorKind,
     message: String,
@@ -21,13 +24,10 @@ pub struct Error {
 
 pub type Result<T> = StdResult<T, Error>;
 
-impl Display for Error {
-    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
-        self.message.fmt(f)
-    }
-}
-
 impl Error {
+    pub const DESC_NO_INTERNET: &'static str =
+        "\n\nAn internet connection is required to download pages for the first time.";
+
     pub const DESC_AUTO_UPDATE_ERR: &'static str =
         "\n\nAn error occurred during the automatic update. \
         To skip updating the cache, run tldr with --offline.";
@@ -57,7 +57,7 @@ impl Error {
     where
         T: Display,
     {
-        self.message = format!("{self} {description}");
+        self.message = format!("{} {description}", self.message);
         self
     }
 
@@ -75,9 +75,14 @@ impl Error {
         Error::new("could not parse the checksum file").kind(ErrorKind::Download)
     }
 
-    pub fn desc_page_does_not_exist() -> String {
+    pub fn desc_page_does_not_exist(try_update: bool) -> String {
+        let e = if try_update {
+            "Try running 'tldr --update'.\n\n"
+        } else {
+            "\n\n"
+        };
         format!(
-            "Try running 'tldr --update'.\n\n\
+            "{e}\
             If the page does not exist, you can create an issue here:\n\
             {}\n\
             or document it yourself and create a pull request here:\n\
@@ -101,7 +106,7 @@ impl Error {
 
     /// Print the error message to stderr and return an appropriate `ExitCode`.
     pub fn exit_code(self) -> ExitCode {
-        let _ = writeln!(io::stderr(), "{} {self}", "error:".red().bold());
+        error!("{}", self.message);
 
         match self.kind {
             ErrorKind::Other | ErrorKind::Io => 1,
